package xyz.lychee.lagfixer.libs.adventure.text.minimessage.internal.parser;

import java.util.Arrays;
import xyz.lychee.lagfixer.libs.adventure.text.minimessage.ParsingException;
import xyz.lychee.lagfixer.libs.annotations.NotNull;
import xyz.lychee.lagfixer.libs.annotations.Nullable;
import xyz.lychee.lagfixer.libs.annotations.ApiStatus.Internal;

@Internal
public class ParsingExceptionImpl extends ParsingException {
   private static final long serialVersionUID = 2507190809441787202L;
   private final String originalText;
   private Token @NotNull [] tokens;

   public ParsingExceptionImpl(final String message, final @Nullable String originalText, final @NotNull Token... tokens) {
      super(message, (Throwable)null, true, false);
      this.tokens = tokens;
      this.originalText = originalText;
   }

   public ParsingExceptionImpl(final String message, final @Nullable String originalText, final @Nullable Throwable cause, final boolean withStackTrace, final @NotNull Token... tokens) {
      super(message, cause, true, withStackTrace);
      this.tokens = tokens;
      this.originalText = originalText;
   }

   public String getMessage() {
      String arrowInfo = this.tokens().length != 0 ? "\n\t" + this.arrow() : "";
      String messageInfo = this.originalText() != null ? "\n\t" + this.originalText() + arrowInfo : "";
      return super.getMessage() + messageInfo;
   }

   public @Nullable String detailMessage() {
      return super.getMessage();
   }

   public @Nullable String originalText() {
      return this.originalText;
   }

   public @NotNull Token @NotNull [] tokens() {
      return this.tokens;
   }

   public void tokens(final @NotNull Token @NotNull [] tokens) {
      this.tokens = tokens;
   }

   private String arrow() {
      Token[] ts = this.tokens();
      char[] chars = new char[ts[ts.length - 1].endIndex()];
      int i = 0;
      Token[] var4 = ts;
      int var5 = ts.length;

      for(int var6 = 0; var6 < var5; ++var6) {
         Token t = var4[var6];
         Arrays.fill(chars, i, t.startIndex(), ' ');
         chars[t.startIndex()] = '^';
         if (Math.abs(t.startIndex() - t.endIndex()) > 1) {
            Arrays.fill(chars, t.startIndex() + 1, t.endIndex() - 1, '~');
         }

         chars[t.endIndex() - 1] = '^';
         i = t.endIndex();
      }

      return new String(chars);
   }

   public int startIndex() {
      return this.tokens.length == 0 ? -1 : this.tokens[0].startIndex();
   }

   public int endIndex() {
      return this.tokens.length == 0 ? -1 : this.tokens[this.tokens.length - 1].endIndex();
   }
}
